﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;
using ComputersShopContracts.BindingModels;
using ComputersShopContracts.ViewModels;
using ComputersShopDataModels;

namespace ComputersShopFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; }
        public string Address { get; private set; }
        public DateTime DateOpen { get; private set; }
        public int MaxCapacity { get; private set; }
        public Dictionary<int, int> Computers { get; private set; } = new();
        private Dictionary<int, (IComputerModel, int)>? _shopComputers = null;
        public Dictionary<int, (IComputerModel, int)> ShopComputers
        {
            get
            {
                if (_shopComputers == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopComputers = Computers.ToDictionary(x => x.Key, y =>
                    ((source.Computers.FirstOrDefault(z => z.Id == y.Key) as IComputerModel)!,
                    y.Value));
                }
                return _shopComputers;
            }
        }

        public static Shop? Create(ShopBindingModel model)
        {
            if (model == null)
                return null;
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                DateOpen = model.DateOpen,
                MaxCapacity = model.MaxCapacity,
                Computers = model.ShopComputers.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }

        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Address = element.Element("Address")!.Value,
                MaxCapacity = Convert.ToInt32(element.Element("MaxCapacity")!.Value),
                DateOpen = Convert.ToDateTime(element.Element("DateOpen")!.Value),
                Computers =
           element.Element("ShopComputers")!.Elements("ShopComputer")
            .ToDictionary(x =>
           Convert.ToInt32(x.Element("Key")?.Value), x =>
           Convert.ToInt32(x.Element("Value")?.Value))
            };
        }


        public void Update(ShopBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Address = model.Address;
            DateOpen = model.DateOpen;
            MaxCapacity = model.MaxCapacity;
            if (model.ShopComputers.Count > 0)
            {
                Computers = model.ShopComputers.ToDictionary(x => x.Key, x => x.Value.Item2);
                _shopComputers = null;
            }
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            DateOpen = DateOpen,
            MaxCapacity = MaxCapacity,
            ShopComputers = ShopComputers
        };

        public XElement GetXElement => new("Shop",
             new XAttribute("Id", Id),
             new XElement("ShopName", ShopName),
             new XElement("Address", Address),
             new XElement("DateOpen", DateOpen),
             new XElement("MaxCapacity", MaxCapacity),
             new XElement("ShopComputers", Computers
                 .Select(x => new XElement("ShopComputer",
                     new XElement("Key", x.Key),
                     new XElement("Value", x.Value))
                 ).ToArray()));

    }
}
